@@ -6,7 +6,7 @@ use core::{
 };
 use ethers_core::{types::H160, utils::to_checksum};
 use http::uri::{Authority, InvalidUri};
-use iri_string::types::{UriAbsoluteString, UriString};
+use iri_string::types::UriString;
 use thiserror::Error;
 
 type TimeStamp = DateTime<Utc>;
@@ -27,11 +27,12 @@ impl FromStr for Version {
     }
 }
 
+#[derive(Clone)]
 pub struct Message {
     pub domain: Authority,
     pub address: [u8; 20],
     pub statement: String,
-    pub uri: UriAbsoluteString,
+    pub uri: UriString,
     pub version: Version,
     pub chain_id: String,
     pub nonce: String,
@@ -45,7 +46,7 @@ pub struct Message {
 impl Display for Message {
     fn fmt(&self, f: &mut Formatter<'_>) -> Result<(), fmt::Error> {
         writeln!(f, "{}{}", &self.domain, PREAMBLE)?;
-        writeln!(f, "{}", to_checksum(&H160(self.address.into()), None))?;
+        writeln!(f, "{}", to_checksum(&H160(self.address), None))?;
         writeln!(f, "\n{}\n", &self.statement)?;
         writeln!(f, "{}{}", URI_TAG, &self.uri)?;
         writeln!(f, "{}{}", VERSION_TAG, self.version as u64)?;
@@ -98,9 +99,9 @@ fn tagged<'a>(tag: &'static str, line: Option<&'a str>) -> Result<&'a str, Parse
         .ok_or(ParseError::Format(tag))
 }
 
-fn parse_line<'a, S: FromStr<Err = E>, E: Into<ParseError>>(
+fn parse_line<S: FromStr<Err = E>, E: Into<ParseError>>(
     tag: &'static str,
-    line: Option<&'a str>,
+    line: Option<&str>,
 ) -> Result<S, ParseError> {
     tagged(tag, line).and_then(|s| S::from_str(s).map_err(|e| e.into()))
 }
@@ -109,7 +110,7 @@ fn tag_optional<'a>(
     tag: &'static str,
     line: Option<&'a str>,
 ) -> Result<Option<&'a str>, ParseError> {
-    match tagged(tag, line).map(|s| Some(s)) {
+    match tagged(tag, line).map(Some) {
         Err(ParseError::Format(t)) if t == tag => Ok(None),
         r => r,
     }
@@ -119,7 +120,7 @@ impl FromStr for Message {
     type Err = ParseError;
     fn from_str(s: &str) -> Result<Self, Self::Err> {
         use hex::FromHex;
-        let mut lines = s.split("\n");
+        let mut lines = s.split('\n');
         let domain = lines
             .next()
             .and_then(|preamble| preamble.strip_suffix(PREAMBLE))
@@ -143,7 +144,7 @@ impl FromStr for Message {
         let mut line = lines.next();
         let expiration_time = match tag_optional(EXP_TAG, line)? {
             Some(exp) => {
-                TimeStamp::from_str(&exp)?;
+                TimeStamp::from_str(exp)?;
                 line = lines.next();
                 Some(exp.into())
             }
@@ -197,10 +198,12 @@ pub enum VerificationError {
     Serialization(#[from] fmt::Error),
     #[error("Recovered key does not match address")]
     Signer,
+    #[error("Message is not currently valid")]
+    Time,
 }
 
 impl Message {
-    pub fn verify_eip191(&self, sig: [u8; 65]) -> Result<Vec<u8>, VerificationError> {
+    pub fn verify_eip191(&self, sig: &[u8; 65]) -> Result<Vec<u8>, VerificationError> {
         use k256::{
             ecdsa::{
                 recoverable::{Id, Signature},
@@ -224,18 +227,31 @@ impl Message {
         }
     }
 
+    pub fn verify(&self, sig: [u8; 65]) -> Result<Vec<u8>, VerificationError> {
+        if !self.valid_now() {
+            Err(VerificationError::Time)
+        } else {
+            self.verify_eip191(&sig)
+        }
+    }
+
     pub fn valid_now(&self) -> bool {
-        let now = Utc::now();
+        self.valid_at(&Utc::now())
+    }
+
+    pub fn valid_at(&self, t: &TimeStamp) -> bool {
         self.not_before
             .as_ref()
             .and_then(|s| TimeStamp::from_str(s).ok())
-            .map(|nbf| now >= nbf)
+            .as_ref()
+            .map(|nbf| t >= nbf)
             .unwrap_or(true)
             && self
                 .expiration_time
                 .as_ref()
                 .and_then(|s| TimeStamp::from_str(s).ok())
-                .map(|exp| now < exp)
+                .as_ref()
+                .map(|exp| t < exp)
                 .unwrap_or(true)
     }
 
@@ -253,17 +269,17 @@ impl Message {
     }
 }
 
-const PREAMBLE: &'static str = " wants you to sign in with your Ethereum account:";
-const ADDR_TAG: &'static str = "0x";
-const URI_TAG: &'static str = "URI: ";
-const VERSION_TAG: &'static str = "Version: ";
-const CHAIN_TAG: &'static str = "Chain ID: ";
-const NONCE_TAG: &'static str = "Nonce: ";
-const IAT_TAG: &'static str = "Issued At: ";
-const EXP_TAG: &'static str = "Expiration Time: ";
-const NBF_TAG: &'static str = "Not Before: ";
-const RID_TAG: &'static str = "Request ID: ";
-const RES_TAG: &'static str = "Resources:";
+const PREAMBLE: &str = " wants you to sign in with your Ethereum account:";
+const ADDR_TAG: &str = "0x";
+const URI_TAG: &str = "URI: ";
+const VERSION_TAG: &str = "Version: ";
+const CHAIN_TAG: &str = "Chain ID: ";
+const NONCE_TAG: &str = "Nonce: ";
+const IAT_TAG: &str = "Issued At: ";
+const EXP_TAG: &str = "Expiration Time: ";
+const NBF_TAG: &str = "Not Before: ";
+const RID_TAG: &str = "Request ID: ";
+const RES_TAG: &str = "Resources:";
 
 #[cfg(test)]
 mod tests {
@@ -326,8 +342,32 @@ Issued At: 2021-12-07T18:28:18.807Z"#,
         )
         .unwrap();
         let correct = <[u8; 65]>::from_hex(r#"6228b3ecd7bf2df018183aeab6b6f1db1e9f4e3cbe24560404112e25363540eb679934908143224d746bbb5e1aa65ab435684081f4dbb74a0fec57f98f40f5051c"#).unwrap();
-        assert!(message.verify_eip191(correct).is_ok());
+        assert!(message.verify_eip191(&correct).is_ok());
         let incorrect = <[u8; 65]>::from_hex(r#"7228b3ecd7bf2df018183aeab6b6f1db1e9f4e3cbe24560404112e25363540eb679934908143224d746bbb5e1aa65ab435684081f4dbb74a0fec57f98f40f5051c"#).unwrap();
-        assert!(message.verify_eip191(incorrect).is_err());
+        assert!(message.verify_eip191(&incorrect).is_err());
+    }
+
+    #[test]
+    fn validation1() {
+        let message = Message::from_str(r#"localhost wants you to sign in with your Ethereum account:
+0x4b60ffAf6fD681AbcC270Faf4472011A4A14724C
+
+Allow localhost to access your orbit using their temporary session key: did:key:z6Mktud6LcDFb3heS7FFWoJhiCafmUPkCAgpvJLv5E6fgBJg#z6Mktud6LcDFb3heS7FFWoJhiCafmUPkCAgpvJLv5E6fgBJg
+
+URI: did:key:z6Mktud6LcDFb3heS7FFWoJhiCafmUPkCAgpvJLv5E6fgBJg#z6Mktud6LcDFb3heS7FFWoJhiCafmUPkCAgpvJLv5E6fgBJg
+Version: 1
+Chain ID: 1
+Nonce: PPrtjztx2lYqWbqNs
+Issued At: 2021-12-20T12:29:25.907Z
+Expiration Time: 2021-12-20T12:44:25.906Z
+Resources:
+- kepler://bafk2bzacecn2cdbtzho72x4c62fcxvcqj23padh47s5jyyrv42mtca3yrhlpa#put
+- kepler://bafk2bzacecn2cdbtzho72x4c62fcxvcqj23padh47s5jyyrv42mtca3yrhlpa#del
+- kepler://bafk2bzacecn2cdbtzho72x4c62fcxvcqj23padh47s5jyyrv42mtca3yrhlpa#get
+- kepler://bafk2bzacecn2cdbtzho72x4c62fcxvcqj23padh47s5jyyrv42mtca3yrhlpa#list"#).unwrap();
+        let correct = <[u8; 65]>::from_hex(r#"20c0da863b3dbfbb2acc0fb3b9ec6daefa38f3f20c997c283c4818ebeca96878787f84fccc25c4087ccb31ebd782ae1d2f74be076a49c0a8604419e41507e9381c"#).unwrap();
+        assert!(message.verify_eip191(&correct).is_ok());
+        let incorrect = <[u8; 65]>::from_hex(r#"30c0da863b3dbfbb2acc0fb3b9ec6daefa38f3f20c997c283c4818ebeca96878787f84fccc25c4087ccb31ebd782ae1d2f74be076a49c0a8604419e41507e9381c"#).unwrap();
+        assert!(message.verify_eip191(&incorrect).is_err());
     }
 }
